SystemOrganization addCategory: #'Neo-JSON-VAST-Core'!

!NeoJSONObjectMapping class methodsFor: '*Neo-JSON-VAST-Core' stamp: 'MarianoMartinezPeck 1/30/2018 10:06'!
new
	^super new initialize! !

!NeoJSONWriter class methodsFor: '*Neo-JSON-VAST-Core' stamp: 'MarianoMartinezPeck 1/30/2018 10:08'!
new
	^super new initialize! !

!Symbol methodsFor: '*Neo-JSON-VAST-Core' stamp: 'marianopeck 1/31/2018 09:13'!
asMutator
	"Return a setter message from a getter message. For example,
	#name asMutator returns #name:
	return self if it is already a setter"
	(self size > 0 and: [self last == $:]) ifTrue:[ ^ self ].
	^ (self copyWith: $:) asSymbol! !

!KeyedCollection methodsFor: '*Neo-JSON-VAST-Core' stamp: 'MarianoMartinezPeck 1/30/2018 09:42'!
neoJsonOn: neoJSONWriter
	neoJSONWriter writeMap: self! !

!NeoJSONObject class methodsFor: '*Neo-JSON-VAST-Core' stamp: 'marianopeck 1/31/2018 11:23'!
newFrom: aDictOrCollection 
	"In Pharo, NeoJSONObject would inherit the implementation from Dictionary, which will work correctly for aDictOrCollection. In VAST, we do not have #newFrom, not all collections understand #associationsDo:  so we implement a custom solution that works for dictionaries and collections with a simply IF. To be improved later. "
	| newDictionary block |
	newDictionary := self new: aDictOrCollection size.
	block := [:x |
		(newDictionary includesKey: x key)
			ifTrue: [self error: 'Duplicate key: ', x key printString]
			ifFalse: [newDictionary add: x]].
	aDictOrCollection isDictionary 
		ifTrue: [ aDictOrCollection associationsDo:  block ] 
		ifFalse: [ aDictOrCollection do: block ].		
	^ newDictionary
! !

!SequenceableCollection methodsFor: '*Neo-JSON-VAST-Core' stamp: 'marianopeck 1/31/2018 12:15'!
allButLastDo: block

	1 to: self size - 1 do:
		[:index | block value: (self at: index)]! !

String variableByteSubclass: #StringAutoConvertableToDBString
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-JSON-VAST-Core'!

!StringAutoConvertableToDBString methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 2/14/2018 10:29'!
at: index put: newElement
	"Same as super implementation with the only difference that if we have a primitive failure and @newElement is not 1 byte character, then
	 we try to auto-convert to DBString instance. This would allow for example using write/read streams with a String and then, on demand, 
	 if the character to read/write is double byte, then simply convert/adapt the stream. For example:
	 
	 StringAutoConvertableToDBString streamContents: [ :out | out nextPut: (Character codePoint: 12353) ].

	would answer a DBString instance. 
	"

	<primitive: VMprStringBasicAtPut> 
	self primitiveErrorCode = PrimErrReadOnly ifTrue: [
		^ self basicAt: index put: newElement ]. 
	newElement codePoint < 256  ifFalse: [	
		"Convert to DBString"
		self become: (self asDBString).
		^self at: index put: newElement.
	].
	^ self primitiveFailed! !

!StringAutoConvertableToDBString methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 2/14/2018 10:16'!
replaceFrom: start to: stop with: aString startingAt: repStart

	"Same as super implementation with the only difference that if we have a primitive failure and @aString is not 1 byte character, then
	 we try to auto-convert to DBString instance. This would allow for example using write/read streams with a String and then, on demand, 
	 if the character to read/write is double byte, then simply convert/adapt the stream. For example:
	 
	 StringAutoConvertableToDBString streamContents: [ :out | out nextPut: (Character codePoint: 12353) ].

	would answer a DBString instance. 
	"

	<primitive: VMprBytesReplaceFromToWithStartingAt>
	^aString class == DBString
		ifTrue: [
			self become: (self asDBString).
			self replaceFrom: start to: stop with: aString startingAt: repStart]
		ifFalse: [super replaceFrom: start to: stop with: aString startingAt: repStart]! !

!Character class methodsFor: '*Neo-JSON-VAST-Core' stamp: ''!
backspace
	"Answer the Character representing a backspace."

	^self value: 8! !

!Character class methodsFor: '*Neo-JSON-VAST-Core' stamp: ''!
newPage
	"Answer the Character representing a form feed."

	^self value: 12! !

!Integer methodsFor: '*Neo-JSON-VAST-Core' stamp: 'marianopeck 1/31/2018 15:59'!
printOn: aStream base: b nDigits: n 

	self printOn: aStream base: b showRadix: false padTo: n with: $0! !

!NeoJSONStreamingWriter class methodsFor: '*Neo-JSON-VAST-Core' stamp: 'MarianoMartinezPeck 1/30/2018 10:08'!
new
	^super new initialize! !

!NeoJSONReader class methodsFor: '*Neo-JSON-VAST-Core' stamp: 'MarianoMartinezPeck 1/30/2018 10:07'!
new
	^super new initialize! !

!Collection methodsFor: '*Neo-JSON-VAST-Core' stamp: 'marianopeck 1/31/2018 12:13'!
ifEmpty: aBlock
	"Evaluate the given block, answering its value if the receiver is empty, otherwise answer the receiver."
	
	"Note that the fact that this method returns its receiver in case the receiver is not empty allows one to write expressions like the following ones: self classifyMethodAs: (myProtocol ifEmpty: ['As yet unclassified'])"

	^ self isEmpty 
		ifTrue: [ aBlock value ]
		ifFalse: [ self ]! !

!Collection methodsFor: '*Neo-JSON-VAST-Core' stamp: 'marianopeck 1/31/2018 09:14'!
removeAllSuchThat: aBlock 
	^self removeAll: (self select: aBlock)! !

!WriteStream methodsFor: '*Neo-JSON-VAST-Core' stamp: 'marianopeck 1/31/2018 09:37'!
<< anObject
	"A more readable, shorter alternative to #nextPutAll: that also
	accepts non-Collection arguments"

	anObject class == collection class
		ifTrue: [ self nextPutAll: anObject ]
		ifFalse: [ anObject putOn: self ]! !

!EsString methodsFor: '*Neo-JSON-VAST-Core' stamp: 'marianopeck 1/31/2018 12:57'!
neoJsonOn: neoJSONWriter
	neoJSONWriter writeString: self! !
